@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/viced/host.c,v 1.57.2.58 2008/02/25 20:39:04 shadow Exp $");
+    ("$Header: /cvs/openafs/src/viced/host.c,v 1.57.2.60 2008/10/27 23:54:10 shadow Exp $");
 
 #include <stdio.h>
 #include <errno.h>
@@ -1104,7 +1104,8 @@ void
 h_Enumerate_r(int (*proc) (), struct host *enumstart, char *param)
 {
     register struct host *host, *next;
-    register int held, nheld;
+    int held = 0;
+    int nheld = 0;
 
     if (hostCount == 0) {
 	return;
@@ -1133,7 +1134,8 @@ hashInsertUuid_r(struct afsUUID *uuid, struct host *host)
 
     /* don't add the same entry multiple times */
     for (chain = hostUuidHashTable[index]; chain; chain = chain->next) {
-	if (host->interface && afs_uuid_equal(&host->interface->uuid, uuid))
+	if (chain->hostPtr->interface && 
+	    afs_uuid_equal(&chain->hostPtr->interface->uuid, uuid))
 	    return;
     }
 
@@ -1946,9 +1948,9 @@ struct client *
 h_FindClient_r(struct rx_connection *tcon)
 {
     register struct client *client;
-    register struct host *host;
+    struct host *host = NULL;
     struct client *oldClient;
-    afs_int32 viceid;
+    afs_int32 viceid = 0;
     afs_int32 expTime;
     afs_int32 code;
     int authClass;
@@ -3031,7 +3033,7 @@ int
 hashDelete_r(afs_uint32 addr, afs_uint16 port, struct host *
 				host)
 {
-    int flag;
+    int flag = 0;
     register struct h_AddrHashChain **hp, *th;
 
     for (hp = &hostAddrHashTable[h_HashIndex(addr)]; (th = *hp);) {
